#!/bin/bash

# This script run a command through SGE

if [ $# -lt 3 ] && [ "$2" != "-waitfor" ] ; then
	echo ""
	echo "USAGE:    manage_task.sh config_file command log_name [options]"
	echo "                         config_file -waitfor job_id"
	echo ""
	echo "OPTIONS:  1) all SGE options"
	echo "          2) -wait   wait until the job is finished"
	echo "             -fast   run the scipt on a fast computer (list of them is specified in a config_file)"
	echo "             -jid    print a job ID to the standard output"
	echo ""
	exit 1
fi


conf=$1
conf_path=${conf%\/*}
conf_global="$conf_path/_global.sh"

source $conf_global
source $conf

echo  "  global config: $conf_global"
echo  "  config:        $conf"

comm=$2
log_name=$3
opt=$4

# waiting for a job termination
if [ "$comm" == "-waitfor" ] ; then
	job_id="$3"
	if [ "$job_id" == "" ] ; then
		echo "ERROR - manage_task.sh - invalid job ID: $job_id"
		exit 1
	fi
	if [ "$job_id" != "local" ] ; then
		while qstat -j $job_id >& /dev/null ; do
			sleep 10s
		done
	fi
#echo ---------------------------------- $job_id
#	qstat -u schwarzp
#echo ----------------------------------
	exit 0
fi

# extract the -wait switch from SGE arguments
param=`echo $opt | sed 's/-wait/ /g'`
param=`echo $param | sed 's/-fast/ /g'`
param=`echo $param | sed 's/-jid/ /g'`


if [ "$use_sge" == "1" ] ; then
	if [ "$sge_email" != "" ] ; then
		if [ "`echo $opt | sed -n '/-M / p'`" == "" ] ; then
			param="$param -M $sge_email"
		fi 
        	if [ "`echo $opt | sed -n '/-m / p'`" == "" ] ; then
			param="$param -m ea"
		fi 
	fi
	
	if [ "`echo $opt | sed -n '/-fast/ p'`" != "" ] ; then
		if [ "$sge_fast_computers" != "" ] && [ "`echo $opt | sed -n '/-q / p'`" == "" ] ; then
			param="$param -q $sge_fast_computers"
		fi
	else
		if [ "$sge_computers" != "" ] && [ "`echo $opt | sed -n '/-q / p'`" == "" ] ; then
			param="$param -q $sge_computers"
		fi
	fi
	
	if [ "$sge_resource" != "" ] && [ "`echo $opt | sed -n '/-l / p'`" == "" ] ; then
		param="$param -l $sge_resource"
	fi

	if [ "$out_dir" == "" ] ; then
		out_dir=`pwd`
	fi
	
	if [ "$exp_dir" == "" ] ; then
		exp_dir=${comm%.*}
		exp_dir=${exp_dir##*/}
	fi	

	mkdir -p $out_dir/sge_log
	touch $out_dir/sge_log/normdir
	echo \#!/bin/bash > /tmp/$$.sge
	echo "$comm" >> /tmp/$$.sge

	# these variables have to be unset else thay replace true variables in the submited job 
	unset JOB_ID
	unset SGE_TASK_ID
	unset SGE_TASK_FIRST
	unset SGE_TASK_LAST
	unset SGE_TASK_STEP
	unset JOB_NAME
	unset HOSTNAME
	
	# add task_id to the log file names 
	task=""
	if [ "`echo $opt | sed -n '/-t / p'`" != "" ] ; then
		task="_\$TASK_ID"
	fi 

	job_name=${exp_dir//\//_}

	#echo "qsub -p -10 -cwd -V -N $job_name -o $out_dir/sge_log/${log_name}${task}_out.log -e $out_dir/sge_log/${log_name}${task}_err.log $param /tmp/$$.sge | tr '.' ' ' | awk '{print $3}'"

	job_id=`qsub -cwd -V -N $job_name -o $out_dir/sge_log/${log_name}${task}_out.log -e $out_dir/sge_log/${log_name}${task}_err.log $param /tmp/$$.sge | tr '.' ' ' | awk '{print $3}'`

	# report the job ID
	if [ "`echo $opt | sed -n '/-jid/ p'`" != "" ] ; then
		echo "$job_id"
	fi

	#echo ------------------- >>  /net/matylda/schwarzp/aa.sge
	#cat /tmp/$$.sge
	#>> /net/matylda/schwarzp/aa.sge
	rm /tmp/$$.sge
	
	# wait until the job is finished
	if [ "`echo $opt | sed -n '/-wait/ p'`" != "" ] ; then
		while qstat -j $job_id >& /dev/null ; do
			sleep 10s
		done
	fi    
else
	# run the command locally
	$comm
	
	# report the job ID
	if [ "`echo $opt | sed -n '/-jid/ p'`" != "" ] ; then
		echo "local"
	fi
fi

