#!/bin/bash

if [ $# -lt 6 ]; then
	echo "This scripts runs SVite on lop files and produce PER"
	echo ""
	echo "USAGE: scoring.sh config_file train_list test_list cv_list mlf phoneme_list output_directory"
	exit 1
fi

# command line parsing
conf=$1

conf_path=${conf%\/*}
conf_global="$conf_path/_global.sh"

source $conf_global
source $conf

train_list1=$2
test_list1=$3
cv_list1=$4
mlf0=$5
phoneme_list=$6
out_dir1=$7

# be sure that VTLN has state level transcription
if [ "$vtln_schedule" = "1" ] ; then
	model_align=1
fi

#test_list1="$work_dir/scoring/tmp/test_lop.scp"
#train_list1="$work_dir/scoring/tmp/train_lop.scp"
#cv_list1="$work_dir/scoring/tmp/cv_lop.scp"
#mlf0="$pfile_dir/mlfs/ref.mlf"
#phoneme_list="$pfile_dir/dicts/dict"
#out_dir1="$work_dir"

# predefined variables
penalty_interval="-20 20"
default_penalty=-4

# create dicectory for scoring if not exist
mkdir -p ${out_dir1}/scoring

# check if all neccessary files exist
if [ ! -e $test_list1 ] || [ ! -e $mlf1 ] || [ ! -e $phoneme_list ] ; then
	echo "Some file does not exist";
	exit
fi

# output file
results="$out_dir1/results.txt"
rm -f $results

# load configuration files
conf_path=${conf%\/*}
conf_global="$conf_path/_global.sh"

source $conf_global
source $conf

# enable model alignment if VTLN
if [ "$do_tr_vtln" = "1" ] || [ "$do_te_vtln" = "1" ] ; then
	model_align=1
fi

# if the list contains states then create a phoneme list and a phoneme MLF
nstates=`cat $phoneme_list | grep "__2" | wc -l`
if [ $nstates == 0 ] ; then
	phoneme_list0=$phoneme_list
	mlf1=$mlf0
else
	phoneme_list0="${out_dir1}/scoring/phn0"

	cat $phoneme_list | sed 's/\(.*\)__[0-9]*$/\1/' | uniq > $phoneme_list0

	mlf1="${out_dir1}/scoring/phn.mlf"
	${script_dir}/misc/mlf_st2phn.pl $mlf0 > $mlf1
fi

# remove the 'oth' phoneme from phoneme list and state list
phoneme_list1=${out_dir1}/scoring/phn
awk '{ if($1 != "oth") print $1 }' $phoneme_list0 > $phoneme_list1

state_list1=${out_dir1}/scoring/states
awk '{ if($1 != "oth") print $1 }' $phoneme_list > $state_list1

# phoneme index - two column list - index, phoneme
awk '{ print NR - 1 " " $1 }' $state_list1 > ${out_dir1}/scoring/state.idx

# list of train label files
mkdir -p ${out_dir1}/scoring/tmp
cat ${train_list1} | tr '/' ' ' | awk '{print  $(NF)}' | sed 's/\(.*\)\.\(.*\)$/\1\.lab/' >  ${out_dir1}/scoring/tmp/train_lab_full.scp
cat ${test_list1} | tr '/' ' ' | awk '{print  $(NF)}' | sed 's/\(.*\)\.\(.*\)$/\1\.lab/' >  ${out_dir1}/scoring/tmp/test_lab.scp
cat ${cv_list1} | tr '/' ' ' | awk '{print  $(NF)}' | sed 's/\(.*\)\.\(.*\)$/\1\.lab/' >  ${out_dir1}/scoring/tmp/cv_lab.scp

# cv set is a part of the training list -> has to be removed
${script_dir}/misc/subtr_lists.pl ${out_dir1}/scoring/tmp/train_lab_full.scp ${out_dir1}/scoring/tmp/cv_lab.scp > ${out_dir1}/scoring/tmp/train_lab.scp

# select only the train part of MLF
echo > ${out_dir1}/scoring/tmp/empty
${bin_dir}/HLEd -l '*' -I $mlf1 -i ${out_dir1}/scoring/tmp/train.mlf -S ${out_dir1}/scoring/tmp/train_lab.scp ${out_dir1}/scoring/tmp/empty

# create lexicon
# it looks like:  phn1 phn1
#                 phn2 phn2
#              or
#                 phn1 phn1_st1 phn1_st2 phn1_st3
#                 phn2 phn2_st1 phn2_st2 phn2_st3
lexicon="${out_dir1}/scoring/lex"
#awk '{split($0, a, "__"); labels[a[1]] = labels[a[1]] " " $1} END{for(l in labels) {print l " " labels[l]}}' $state_list1 > $lexicon
awk '{ split($0, a, /__[0-9]/); labels[a[1]]=labels[a[1]] " " $0} END {for (l in labels) { print l,labels[l]}}' $state_list1 > $lexicon

# compute bigram LM
mkdir -p ${out_dir1}/lm
${bin_dir}/HLStats -o -b ${out_dir1}/lm/bigram.lm -S ${out_dir1}/scoring/tmp/train_lab.scp -I ${out_dir1}/scoring/tmp/train.mlf $phoneme_list1

# create language nets
${script_dir}/misc/create_net.pl -o ${out_dir1}/scoring/monophones_lnet -p $phoneme_list1 -l $lexicon
${script_dir}/misc/create_net.pl -o ${out_dir1}/scoring/monophones_lnet_ngram -a ${out_dir1}/lm/bigram.lm -l $lexicon

# create models for SVite
${script_dir}/misc/create_dummy_hmms.pl $state_list1 > ${out_dir1}/scoring/hmmdefs_dummy
${script_dir}/misc/gmm2like.pl ${out_dir1}/scoring/hmmdefs_dummy ${out_dir1}/scoring/state.idx > ${out_dir1}/scoring/hmmdefs_like_1

# fix the VecSize parameter in model file
# - GMM - 39, TRAPS - 40 (oth phoneme)
pf=`head $cv_list1 -n 1`;                                                                  # get first lop file from the list
np=`${bin_dir}/HList -h $pf | head -n 3 | awk '{ if(NR == 3) print $3 }'`            # dump HTK header and extract VecSize
cat ${out_dir1}/scoring/hmmdefs_like_1 | sed 's/\(.*\)VecSize.*PDFObsVec\(.*\)/\1VecSize> '$np' <PDFObsVec\2/' > ${out_dir1}/scoring/hmmdefs_like
rm ${out_dir1}/scoring/hmmdefs_like_1

# -- scoring without LM
if [ -e $cv_list1 ] ; then
	if [ $tune_on_test_set == 1 ] ; then
		echo "Tuning on the test set !!!"  >> $results
		if [ $tune_max_acc == "1" ] ; then
			echo "The maximal accuracy was tuned !!!"  >> $results
			${script_dir}/misc/tunepenalty_acc.pl $penalty_interval ${out_dir1}/scoring $test_list1 $mlf1 $phoneme_list1 ${out_dir1}/scoring/monophones_lnet 0 $bin_dir >> $results
		else
			${script_dir}/misc/tunepenalty.pl $penalty_interval ${out_dir1}/scoring $test_list1 $mlf1 $phoneme_list1 ${out_dir1}/scoring/monophones_lnet 0 $bin_dir >> $results
		fi
	else
		if [ $tune_max_acc == "1" ] ; then
			echo "The maximal accuracy was tuned !!!"  >> $results
			${script_dir}/misc/tunepenalty_acc.pl $penalty_interval ${out_dir1}/scoring $cv_list1 $mlf1 $phoneme_list1 ${out_dir1}/scoring/monophones_lnet 0 $bin_dir >> $results
		else
			${script_dir}/misc/tunepenalty.pl $penalty_interval ${out_dir1}/scoring $cv_list1 $mlf1 $phoneme_list1 ${out_dir1}/scoring/monophones_lnet 0 $bin_dir >> $results
		fi
	fi

	penalty=`cat $results | grep "Penalty" | tail -n 1 | sed 's/^Penalty=\(.*\)$/\1/'`

	if [ "$penalty" = "" ] ; then
		exit 1
	fi
	echo "" >> $results
else
	penalty=$default_penalty
	echo "Penalty: $penalty" > $results
	echo "" >> $results
fi

# run SVite decoder
${bin_dir}/SVite -S $test_list1 -H ${out_dir1}/scoring/hmmdefs_like -i ${out_dir1}/rec.mlf -l '*' -p $penalty -w ${out_dir1}/scoring/monophones_lnet

if [ "$model_align" = "1" ] ; then
	${bin_dir}/SVite -m -S $test_list1 -H ${out_dir1}/scoring/hmmdefs_like -i ${out_dir1}/rec_m.mlf -l '*' -p $penalty -w ${out_dir1}/scoring/monophones_lnet
fi

if [ "$do_score_tr" = "1" ] ; then
	${bin_dir}/SVite -S $train_list1 -H ${out_dir1}/scoring/hmmdefs_like -i ${out_dir1}/train_rec.mlf -l '*' -p $penalty -w ${out_dir1}/scoring/monophones_lnet

	if [ "$model_align" = "1" ] ; then
		${bin_dir}/SVite -m -S $train_list1 -H ${out_dir1}/scoring/hmmdefs_like -i ${out_dir1}/train_rec_m.mlf -l '*' -p $penalty -w ${out_dir1}/scoring/monophones_lnet
	fi
fi

# calculate PER
if [ -e ${out_dir1}/scoring/cv.mlf ] ; then
	cp ${out_dir1}/scoring/cv.mlf ${out_dir1}/cv.mlf
	echo CV PART >> $results
	${bin_dir}/HResults -p -I $mlf1 $phoneme_list1 ${out_dir1}/cv.mlf  >> $results
fi

echo "" >> $results
echo TEST PART >> $results
${bin_dir}/HResults -p -I $mlf1 $phoneme_list1 ${out_dir1}/rec.mlf  >> $results

if [ -e ${out_dir1}/train_rec.mlf ] ; then
	echo "" >> $results
	echo TRAIN PART >> $results
	${bin_dir}/HResults -p -I $mlf1 $phoneme_list1 ${out_dir1}/train_rec.mlf >> $results
fi


# -- scoring with LM
if [ $score_with_bigram == 1 ] ; then
	echo "" >> $results
	if [ -e $cv_list1 ] ; then
		if [ $tune_on_test_set == 1 ] ; then
			echo "Tuning on the test set !!!"  >> $results
			if [ $tune_max_acc == "1" ] ; then
				echo "The maximal accuracy was tuned !!!"  >> $results
				${script_dir}/misc/tunepenalty_acc.pl $penalty_interval ${out_dir1}/scoring $test_list1 $mlf1 $phoneme_list1 ${out_dir1}/scoring/monophones_lnet_ngram $lmscale $bin_dir >> $results
			else
				${script_dir}/misc/tunepenalty.pl $penalty_interval ${out_dir1}/scoring $test_list1 $mlf1 $phoneme_list1 ${out_dir1}/scoring/monophones_lnet_ngram $lmscale $bin_dir >> $results
			fi

		else
			if [ $tune_max_acc == "1" ] ; then
				echo "The maximal accuracy was tuned !!!"  >> $results
				${script_dir}/misc/tunepenalty_acc.pl $penalty_interval ${out_dir1}/scoring $cv_list1 $mlf1 $phoneme_list1 ${out_dir1}/scoring/monophones_lnet_ngram $lmscale $bin_dir >> $results
			else
				${script_dir}/misc/tunepenalty.pl $penalty_interval ${out_dir1}/scoring $cv_list1 $mlf1 $phoneme_list1 ${out_dir1}/scoring/monophones_lnet_ngram $lmscale $bin_dir >> $results
			fi
		fi
		penalty=`cat $results | grep "Penalty" | tail -n 1 | sed 's/^Penalty=\(.*\)$/\1/'`
		if [ $penalty = "" ] ; then
			exit 1
		fi
		echo "" >> $results
	else
		penalty=$default_penalty
		echo "Penalty: $penalty" > $results
		echo "" >> $results
	fi

	# run SVite decoder
	${bin_dir}/SVite -S $test_list1 -H ${out_dir1}/scoring/hmmdefs_like -i ${out_dir1}/rec_lm.mlf -l '*' -p $penalty -w ${out_dir1}/scoring/monophones_lnet_ngram -s $lmscale

	if [ "$model_align" = "1" ] ; then
		${bin_dir}/SVite -m -S $test_list1 -H ${out_dir1}/scoring/hmmdefs_like -i ${out_dir1}/rec_lm_m.mlf -l '*' -p $penalty -w ${out_dir1}/scoring/monophones_lnet_ngram -s $lmscale
	fi

	if [ "$do_score_tr" = "1" ]; then
		${bin_dir}/SVite -S $train_list1 -H ${out_dir1}/scoring/hmmdefs_like -i ${out_dir1}/train_rec_lm.mlf -l '*' -p $penalty -w ${out_dir1}/scoring/monophones_lnet_ngram -s $lmscale

		if [ "$model_align" = "1" ] ; then
			${bin_dir}/SVite -m -S $train_list1 -H ${out_dir1}/scoring/hmmdefs_like -i ${out_dir1}/train_rec_lm_m.mlf -l '*' -p $penalty -w ${out_dir1}/scoring/monophones_lnet_ngram -s $lmscale
		fi
	fi

	# calculate PER
	if [ -e ${out_dir1}/scoring/cv.mlf ] ; then
		cp ${out_dir1}/scoring/cv.mlf ${out_dir1}/cv_lm.mlf
		echo CV PART with LM >> $results
		${bin_dir}/HResults -p -I $mlf1 $phoneme_list1 ${out_dir1}/cv_lm.mlf  >> $results
	fi

	echo "" >> $results
	echo TEST PART with LM >> $results
	${bin_dir}/HResults -p -I $mlf1 $phoneme_list1 ${out_dir1}/rec_lm.mlf  >> $results

	if [ -e ${out_dir1}/train_rec_lm.mlf ] ; then
		echo "" >> $results
		echo TRAIN PART with LM >> $results
		${bin_dir}/HResults -p -I $mlf1 $phoneme_list1 ${out_dir1}/train_rec_lm.mlf >> $results
	fi
fi
