#!/bin/sh

if [ $# -ne 4 ] ; then
	echo USAGE: streams_forward_pass.sh config_file name input_pfile output_pfile
	exit 1
fi

conf=$1
name=$2
input=$3
output=$4
out_dir=$5

# configuration
conf_path=${conf%\/*}
conf_global="$conf_path/_global.sh"

source $conf_global
source $conf

# check temp
work_dir=$out_dir
if [ $use_tmp == 1 ] ; then
	loc_tmp_dir="$tmp_dir/$$"                                              # /tmp/$user/$process_id
	mkdir -p $loc_tmp_dir
	free_space=`df $loc_tmp_dir | tail -n 1 | awk '{print int($4/1024)}'`  # in MB
	if [ $free_space -ge $tmp_min_space ] ; then
		work_dir=$loc_tmp_dir                                          # there is not enough space in the tmp
	fi
fi

# append full path to the configuration file name if one does not exist one
if [ ${conf:0:1} != "/" ] && [ ${conf:0:1} != "." ] ; then
	conf="`pwd`/$conf"
fi

# number of streams
nstreams=1
while [ "${mfccer[$nstreams]}" != "" ] ; do
	nstreams=$(( $nstreams+1 ))
done
nstreams=$(( $nstreams - 1 ))

# forward pass streams
if [ $do_fwd_bands == 1 ] ; then
	if [ $use_sge == 1 ] && [ $nstreams != 1 ] ; then
		nstreams_1=$(( $nstreams - 1 ))
		jid=`${script_dir}/manage_task.sh $conf "${script_dir}/streams_forward_pass_stream.sh $conf $input $name \\\$SGE_TASK_ID" ${name}_forward_pass "-t 1-$nstreams_1 -jid -m a" | tail -n 1`

		# run one stream on this computer
		${script_dir}/streams_forward_pass_stream.sh $conf $input $name $nstreams

		${script_dir}/manage_task.sh $conf -waitfor $jid
	else
		for (( i=1 ; i <= $nstreams ; i=i+1 )) ; do
			${script_dir}/streams_forward_pass_stream.sh $conf $input $name $i
		done
	fi
fi

if [ $do_fwd_merger == 1 ] ; then
	if [ $use_lin_merging != 1 ] ; then   # merger is NN
		# reference labels for merger - testing pfile
		is_lbl=`${bin_dir}/pfile_info -i $test_tnn | tail -n 1 | awk '{print $5}'`
		labels=""

		if [ $is_lbl != 0 ] ; then
			mkdir -p $work_dir/labels
			${bin_dir}/pfile_print -i $input -ns -fr nil -q > $work_dir/labels/merger_${name}.labs
			labels=$work_dir/labels/merger_${name}.labs
		fi

		# merge rabbin files into one pfile
		mkdir -p $work_dir/pfiles
		fmerged="$work_dir/pfiles/${name}_mnn.pfile"
		fmerged_postpr="$work_dir/pfiles/${name}_mnn_postpr.pfile"

		# merge streams
		${script_dir}/streams_merge_streams.sh $conf $nstreams $out_dir/ffout/${name}_s "_postpr.pfile" $fmerged $labels

		if [ $is_lbl != 0 ] ; then
			rm -f $work_dir/labels/merger_${name}.labs
			rmdir $work_dir/labels
		fi

		# remove temporal files
		if [ $keep_test_band_posteriors != 1 ] && [ $do_fwd_bands == 1 ] ; then
			for (( i=0 ; i < $nstreams ; i=i+1 )) ; do
				rm -f "$out_dir/ffout/${name}_b${i}.pfile"
				rm -f "$out_dir/ffout/${name}_b${i}_postpr.pfile"
			done
			if [ "`ls $out_dir/ffout`" == "" ] ; then
				rmdir $out_dir/ffout
			fi
		fi

		# postprocessing of merged posteriors
		${script_dir}/streams_postprocess_mergerd_streams.sh $conf $fmerged $fmerged_postpr

		# forward pass through the merger
		mkdir -p ${output%/*}
		${script_dir}/fwd_MN.sh $conf ${system}_merger $output $fmerged_postpr
		
		if [ $keep_test_merged_band_posteriors == 1 ] && [ -e $work_dir/pfiles/${name}_mnn.pfile ] ; then
			mkdir -p $out_dir/pfiles
			if [ ! $out_dir/pfiles/${name}_mnn.pfile -ef $work_dir/pfiles/${name}_mnn.pfile ] ; then
				rm -f $out_dir/pfiles/${name}_mnn.pfile
				mv $work_dir/pfiles/${name}_mnn.pfile $out_dir/pfiles/${name}_mnn.pfile
			fi
		else
			rm -f $work_dir/pfiles/test_mnn_par.pfile
		fi
	else                                 # linear merging
		# create list of input files
		input_fs=""
		for (( i=0 ; i < $nstreams ; i=i+1 )) ; do
			file=$out_dir/ffout/${name}_s${i}"_postpr.pfile"
			if [ "$input_fs" != "" ] ; then
				input_fs="$input_fs "
			fi
			input_fs="$input_fs $file"
		done
		mkdir -p ${output%/*}
		if [ "$merging_weights" = "" ] ; then
			merging_weights=`echo | awk '{ print 1/'$nstreams' }'`
		fi
		echo ${bin_dir}/lmergerapbin -l -w "$merging_weights" $output $input_fs
		${bin_dir}/lmergerapbin -l -w "$merging_weights" $output $input_fs
		mkdir -p $out_dir/vars
		echo "posterior probabilities are in logs" > $out_dir/vars/logposteriors
	fi
fi

rm -f $fmerged_postpr
if [ -e $work_dir/pfiles ] && [ "`ls $work_dir/pfiles`" == "" ] ; then
	rmdir $work_dir/pfiles;
fi

if [ $use_tmp == 1 ] ; then
	rm -r -f $loc_tmp_dir
	if [ "`ls $tmp_dir`" == "" ] ; then
		rmdir $tmp_dir
	fi
fi

