#!/bin/sh

if [ $# -ne 2 ] ; then
	echo USAGE: streams_train_stream.sh config_file stream{1-N}
	exit 1
fi

conf=$1
net_idx=$2

# SGE does not work with task_id 0
i=$(( $net_idx - 1 ))

# configuration
conf_path=${conf%\/*}
conf_global="$conf_path/_global.sh"

source $conf_global
source $conf

# path to this script
p=${0%\/*}

# check temp
work_dir=$out_dir
if [ $use_tmp == 1 ] ; then
	loc_tmp_dir="$tmp_dir/$$"                                              # /tmp/$user/$process_id
	mkdir -p $loc_tmp_dir
	free_space=`df $loc_tmp_dir | tail -n 1 | awk '{print int($4/1024)}'`  # in MB
	if [ $free_space -ge $tmp_min_space ] ; then
		work_dir=$loc_tmp_dir                                          # there is not enough space in the tmp
	fi
fi


if [ ! -e $out_dir/weights/${system}_s$i.weights ] || [ ! -e $out_dir/ffout/train_s${i}_postpr.pfile ] ; then

	# extract traps
	mkdir -p $work_dir/pfiles
	if [ ! -e $work_dir/pfiles/train_s${i}.pfile ] ; then
		${script_dir}/param_FN.sh $conf $train_tnn $work_dir/pfiles/train_s${i}.pfile $net_idx
	fi

	mkdir -p $out_dir/ffout

	if [ "${is_net[$net_idx]}" != "0" ] ; then
		# train band classifiers
		if [ ! -e $out_dir/weights/${system}_s$i.weights ] ; then
			${p}/train_FN.sh $conf ${system}_s$i $work_dir/pfiles/train_s${i}.pfile
		fi

		# pass the train set through band classifiers
		if [ ! -e $out_dir/ffout/train_s$i.pfile ] ; then
			${p}/fwd_FN.sh $conf ${system}_s$i $out_dir/ffout/train_s$i.pfile $work_dir/pfiles/train_s${i}.pfile
		fi

		rm -f $work_dir/pfiles/train_s${i}.pfile
	else
		nice -n 19 mv $work_dir/pfiles/train_s${i}.pfile $out_dir/ffout/train_s$i.pfile
	fi

	# postprocess band posteriors
	${p}/streams_postprocess_stream_posteriors.sh $conf $out_dir/ffout/train_s${i}.pfile $out_dir/ffout/train_s${i}_postpr.pfile $i
fi

if [ $use_tmp == 1 ] ; then
	rm -r -f $loc_tmp_dir
	if [ "`ls $tmp_dir`" == "" ] ; then
		rmdir $tmp_dir
	fi
fi
