#!/bin/bash

if [ $# -lt 6 ]; then
	echo "One file VTLN"
	echo ""
	echo "USAGE: vtln.sh config_file tmp_dir source_file target_file label_file hcopy_conf"
	exit 1
fi

# command line arguments
conf=$1
tmp=$2
source_file=$3
target_file=$4
label_file=$5
hcopy_conf=$6

wav=$tmp/wav
mb1=$tmp/mb1
mb2=$tmp/mb2
mfcc1=$tmp/mfcc1
mfcc2=$tmp/mfcc2
mlf=$tmp/mlf
nl=$tmp/nl
fname=$tmp/fname

rm -f $mb1
rm -f $mb2

# source config files
conf_path=${conf%\/*}
conf_global="$conf_path/_global.sh"

source $conf_global
source $conf

# initial two warping factors
wfs=`echo "$vtln_min $vtln_max" | awk '{d = $2 - $1; c = ($1 + $2) / 2; print c - d / 4 " " c + d / 4}'`
wf1=`echo $wfs | awk '{print $1}'`
wf2=`echo $wfs | awk '{print $2}'`

stop=0

if [ ! -e "$source_file" ] ; then
	echo "ERROR: vtln_1file.sh - source waveform file '$source_file' does not exist"
	exit 1
fi

rm -f $wav
cp $source_file $wav

while [ "$stop" == "0" ] ; do

	# parameterization
	cp $hcopy_conf $tmp/hcopy1.cfg
	cp $hcopy_conf $tmp/hcopy2.cfg

	echo "WARPFREQ=$wf1
	      WARPLCUTOFF=$vtln_lfreq
              WARPUCUTOFF=$vtln_ufreq" >> $tmp/hcopy1.cfg
	echo "WARPFREQ=$wf2
	      WARPLCUTOFF=$vtln_lfreq
	      WARPUCUTOFF=$vtln_ufreq" >> $tmp/hcopy2.cfg

	${bin_dir}/HCopy -C $tmp/hcopy1.cfg $wav $mb1
	${bin_dir}/HCopy -C $tmp/hcopy2.cfg $wav $mb2
	${bin_dir}/feacat -i $mb1 -ip htk -o ${mb1}.pfile -op pfile
	${bin_dir}/feacat -i $mb2 -ip htk -o ${mb2}.pfile -op pfile
	${bin_dir}/$vtln_param -i ${mb1}.pfile -o $mfcc1
	${bin_dir}/$vtln_param -i ${mb2}.pfile -o $mfcc2

	if [ ! -e "$label_file" ] ; then
		echo "ERROR: vtln_1file.sh - label file '$label_file' does not exist"
		exit 1
	fi

	# labels
	echo "#!MLF!#" > $mlf
	lab_file_name=${label_file##*/}
	echo "\"*/$lab_file_name\"" >> $mlf
	cat $label_file >> $mlf
	echo "." >> $mlf
	echo $source_file > $fname
	${bin_dir}/pfile_labs -rl $mlf -f $fname -i $mfcc1 -d $pfile_dir/dicts/dict -snl $nl -w

	# calculate likelihood
	like1=`${bin_dir}/pfile_like -i $mfcc1 -s $out_dir/vtln/gmm/models -lf $nl | awk '{print $1}'`
	like2=`${bin_dir}/pfile_like -i $mfcc2 -s $out_dir/vtln/gmm/models -lf $nl | awk '{print $1}'`

	# new warping factors
	it=`echo "$like1 $like2 $wf1 $wf2" | awk '           \
		{                                            \
			dl = ($1 > $2 ? $1 - $2 : $2 - $1);  \
			if(dl > '$vtln_deltalike')           \
				stop = 0;                    \
			else if($1 > $2)                     \
				stop = 1;                    \
			else                                 \
			        stop = 2;                    \
			d = $4 - $3;                         \
			c = ($1 > $2 ? $3 : $4);             \
			wf1 = c - d / 4;                     \
			wf2 = c + d / 4;                     \
			print "IN: wf1=" $3 " wf2=" $4 " l1=" $1 " l2=" $2 " dl=" dl " | OUT: stop=" stop " wf1=" wf1 " wf2=" wf2 " wf=" c \
		}'`

	echo $it

	stop=`echo $it | awk '{print $9}'`
	stop=${stop#*=}
	wf1=`echo $it | awk '{print $10}'`
	wf1=${wf1#*=}
	wf2=`echo $it | awk '{print $11}'`
	wf2=${wf2#*=}
	wf=`echo $it | awk '{print $12}'`
	wf=${wf#*=}
done

dir=${target_file%/*}
mkdir -p $dir

if [ "$stop" = "1" ] ; then
	cp $mb1 $target_file
else
	cp $mb2 $target_file
fi

echo "$wf"
